% invPend.m
%
% Matlab script that is used to design a stabilzing controller for an
% inverted pendulum 
%
% Dynamic Programming and Optimal Control
% Fall 2009
% Exercise in class
%
% --
% ETH Zurich
% Institute for Dynamic Systems and Control
% Angela Schllig
% aschoellig@ethz.ch
%
% --
% Revision history
% [15.12.09, AS]    first version
%
%


% clear workspace and command window
clear;
clc;


%% Parameters
m = 0.1; % mass in [kg]
l = 0.2; % length in [m]
g = 9.81; %gravtational constant in [N/kg]

PARAM.m = m;
PARAM.l = l;
PARAM.g = g;

%% Linearized continous-time state space model
A = [0 1; g/l 0];
B = [0; m/l^2];
C = eye(2);
D = [0; 0];

%% Analyze system
P_c = ss(A,B,C,D);

disp('*Poles')
pole(P_c)
disp('Zeros')
zero(P_c)

%% Sampling time
tau = 0.01;

%% Discretize
M = expm([A, B; zeros(1,3)].*tau);
A_d = M(1:2, 1:2);
B_d = M(1:2,3);

% or use Matlab inbuilt-function
P_d = c2d(ss(A,B,C,D), tau, 'zoh');

%% LQR design
% weight on u
R = 0.00001;

% weight on phi and phidot
q1 = 1; % phi
q2 = 0; % phidot

Q = [q1, 0; 0 q2];

% check controllability
rank(ctrb(A_d,B_d))
% check observability
rank(obsv(A_d,0.5.*Q))

% Solve discrete algebraic Riccati equation: K controller gain, L poles of
% closed loop
[X,L,F] = dare(A_d,B_d,Q,R); 

%% Step response
% Initial state
x0 = [pi/6;0];
CL_d = ss(P_d.a-P_d.b*F,zeros(2,1),P_d.c,P_d.d,tau);

[y,t,x] = initial(CL_d,x0);
u = -F * x';

figure()
subplot(2,1,1)
plot(t, x)
legend('Angle in rad', 'Angular velocity in rad/s', 'Location', 'SouthEast')
xlabel('Time t in s')
ylabel('I in Nm')
title(['R = ',num2str(R),'  q1 = ',num2str(q1),'  q2 = ',num2str(q2)])
grid on

subplot(2,1,2)
plot(t, u)
xlabel('Time t in s')
ylabel('Input u in Nm')
grid on

%% Movie
% x = [0.9*pi 0]'; % state: theta, thetadot
% theta measured from hanging down, counterclockwise
% convert 
xplot(:,1) = pi-x(:,1);
xplot(:,2) = - x(:,2);
for i = 1:length(t)
    drawPend(t(i),xplot(i,:)','Output');
    pause(tau*10)
end
